/******************************************************************************
 *
 * Copyright (c) 2006 Jon Black
 *
 * This file is part of Gibberish.
 *
 * Gibberish is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Gibberish is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *****************************************************************************/

#include <QtDebug>
#include <QSqlQuery>

#include "DatabaseInterface.h"
#include "LessonDialog.h"
#include "Lesson.h"
#include "LessonSettings.h"
#include "Score.h"
#include "Questions.h"
#include "LessonWidget.h"
#include "TimerWidget.h"

// *****************************************************************************
// Constructor/Destructor
// *****************************************************************************

LessonDialog::LessonDialog( QWidget * parent, Lesson * lesson )
	: QDialog( parent )
{
	setupUi( this );

	// TODO Remove this when timer is proper widget
	timerWidget = new TimerWidget;
	timerWidget->setTime( lesson->getLessonSettings()->getTime() );
	qgbTime->layout()->addWidget( timerWidget );

	this->lesson = lesson;

	// Create the correct lesson widget
	switch( lesson->getLessonSettings()->getLessonType() )
	{
		//case 0:
		//	lessonWidget = new LessonWidgetFC;
		//	qDebug() << "Message: Created Flash Card Widget.";
		//	break;
		case 0:
			lessonWidget = new LessonWidgetMC;
			qDebug() << "Message: Created Multiple Choice Widget.";
			break;
		//case 1:
		//	lessonWidget = new LessonWidgetQA;
		//	qDebug() << "Message: Created Question Answer Widget.";
		//	break;
		default:
			qDebug() << "Error. Invalid lesson type. Probably going to crash now.";
			return;
	}

	// Set the instructions
	qtbInstructions->setHtml( lessonWidget->getInstructions() );

	// Now add the LessonWidget to the dialog.
	delete lessonWidgetPage->layout();      // Delete the old layout
	QHBoxLayout *layout = new QHBoxLayout;
  layout->addWidget( lessonWidget );
  lessonWidgetPage->setLayout( layout );

	// Show the lesson title
	qlLessonTitle->setText( this->lesson->getTitle() );

	// WOOOOOOOOOOOOOO CLEVER CLEVER!
	connect( timerWidget, SIGNAL( timeUp() ), this, SLOT( endLesson() ) );
}

// *****************************************************************************
// Methods
// *****************************************************************************

/**
 *
 */
void LessonDialog::on_qpbStart_clicked()
{
	// Setup the components' contents
	QString text1, text2;

	text1 = QString("%1/%2").arg( lesson->getQuestions()->getQuestionNumberCount() ).arg( lesson->getQuestions()->getQuestionNumberTotal() );
	text2 = QString("%1/%2").arg( 0 ).arg( lesson->getQuestions()->getQuestionNumberCount() - 1 );

	qlQuestionNumber->setText( text1 );
	qprQuestionNumber->setRange( 0, lesson->getQuestions()->getQuestionNumberTotal() );
	qprQuestionNumber->setValue( lesson->getQuestions()->getQuestionNumberCount() );
	qlScore->setText( text2 );
	qprScore->setRange( 0, 1 );
	qprScore->setValue( 0 );

	// Enable the lesson controls
	qpbStart->setEnabled( false );
	qpbNext->setEnabled( true );

	// Show the quiz section
	qswLessonStack->setCurrentIndex(1);

	// Show the first question
	on_qpbNext_clicked();
	
	// TODO Start the timer
	if ( lesson->getLessonSettings()->isUseTimer() )
	{
		timerWidget->startTimer();
		qpbPause->setEnabled( true );
	}
}

/**
 *
 */
void LessonDialog::on_qpbNext_clicked()
{
	// Check the answer if there is one
	if ( lesson->getQuestions()->getQuestionNumberCount() > 0 )
	{
		bool correctAnswer = lessonWidget->checkAnswer( lesson->getQuestions() );
		if ( correctAnswer )
			lesson->getScore()->incrementCorrectAnswerCount();
	
		// set the given answer to the question in Score
		lesson->getScore()->addGivenAnswer( lessonWidget->getAnswerGiven() );
	}
	
	// Update score display
	QString text2 = QString("%1/%2").arg( lesson->getScore()->getCorrectAnswerCount() ).arg( lesson->getQuestions()->getQuestionNumberCount() );
	qlScore->setText( text2 );
	qprScore->reset();
	qprScore->setRange( 0, lesson->getQuestions()->getQuestionNumberCount() );
	qprScore->setValue( lesson->getScore()->getCorrectAnswerCount() );

	// Have we asked enough questions?
	if ( lesson->getQuestions()->getQuestionNumberCount() < lesson->getQuestions()->getQuestionNumberTotal() )
	{
		// Display the next question in the LessonWidget
		lessonWidget->nextQuestion( lesson->getQuestions() );

		qDebug() << "Questions Asked:" << lesson->getQuestions()->getQuestionNumberCount();
		qDebug() << "Questions Total:" << lesson->getQuestions()->getQuestionNumberTotal();
		qDebug() << "Questions Left: " << lesson->getQuestions()->getQuestionNumberTotal() - lesson->getQuestions()->getQuestionNumberCount();

		// Update question number display
		QString text1 = QString("%1/%2").arg( lesson->getQuestions()->getQuestionNumberCount() ).arg( lesson->getQuestions()->getQuestionNumberTotal() );
		qlQuestionNumber->setText( text1 );
		qprQuestionNumber->setValue( lesson->getQuestions()->getQuestionNumberCount() );
	}
	else
	{
		endLesson();
	}
}

/**
 *
 */
void LessonDialog::on_qpbPause_clicked()
{
	if ( qswLessonStack->currentIndex() == 1)
	{
		timerWidget->pauseTimer();
		qswLessonStack->setCurrentIndex(2);
		qpbNext->setEnabled( false );
	}
	else
	{
		timerWidget->startTimer();
		qswLessonStack->setCurrentIndex(1);
		qpbNext->setEnabled( true );
	}
}

/**
 *
 */
void LessonDialog::endLesson()
{
	int newResultID;

	// Tell the widget to end
	lessonWidget->endLesson();

	// Update display
	qswLessonStack->setCurrentIndex(3);
	qpbNext->setEnabled( false );
	qpbPause->setEnabled( false );
	
	// Fill summary table with results of the lesson &
  // insert the results into the database
	QList<int> answersGiven = lesson->getScore()->getAnswersGiven();
	QList<int> questionsAsked = lesson->getQuestions()->getQuestionsAsked();
	
	DatabaseInterface dbInterface;
	

	// Generate new id for database table
	QString	sql = QString("SELECT ResultID FROM Results ORDER BY ResultID;");
	QSqlQuery * query = dbInterface.exec( sql );

	if ( query->isActive() ) 
	{
		query->last();
		newResultID = query->value(0).toInt() + 1;
	}

	for ( int i = 0; i < answersGiven.size(); i++ )
	{
		// Insert table item
		QTreeWidgetItem* treeItem = new QTreeWidgetItem( qtwResults );
		treeItem->setText( 0, QString( "%1" ).arg(i+1) );

		treeItem->setText( 1, lesson->getQuestions()->getQuestions().at( questionsAsked.at(i) ) ); // question
		treeItem->setText( 2, lesson->getQuestions()->getAnswers().at( questionsAsked.at(i) ) ); 	// Answer expected
		treeItem->setText( 3, lesson->getQuestions()->getAnswers().at( answersGiven.at(i) ) ); 	// Answer given

		// Insert database record
		int acorrect = 0;
		if ( answersGiven.at(i) == questionsAsked.at(i) )
		{
			treeItem->setBackgroundColor( 0, QColor( 140, 255, 140 ) );
			treeItem->setBackgroundColor( 1, QColor( 140, 255, 140 ) );
			treeItem->setBackgroundColor( 2, QColor( 140, 255, 140 ) );
			treeItem->setBackgroundColor( 3, QColor( 140, 255, 140 ) );
			treeItem->setBackgroundColor( 4, QColor( 140, 255, 140 ) );
			treeItem->setText( 4, "Correct" );
			acorrect = 1;
		}
		else
		{
			treeItem->setBackgroundColor( 0, QColor( 255, 140, 140 ) );
			treeItem->setBackgroundColor( 1, QColor( 255, 140, 140 ) );
			treeItem->setBackgroundColor( 2, QColor( 255, 140, 140 ) );
			treeItem->setBackgroundColor( 3, QColor( 255, 140, 140 ) );
			treeItem->setBackgroundColor( 4, QColor( 255, 140, 140 ) );
			treeItem->setText( 4, "Incorrect" );
		}

		// %1=resultid %2=questionnumber %3=result 
		sql = QString("INSERT INTO Results VALUES('%1','%2','%3');").arg(newResultID).arg(i+1).arg(acorrect);
		dbInterface.exec( sql );
	}

	// Create the entry for this lesson
	// %1=lessonid %2=langid %3=resultid %4=datetime
	QString dateTime = QDateTime::currentDateTime().toString( "dd/MM/yyyy hh:mm" );
	sql = QString("INSERT INTO TakenLessons VALUES('%1','%2','%3','%4');").arg( lesson->getLessonID() ).arg( lesson->getLessonSettings()->getStudyLang() ).arg( newResultID ).arg(dateTime);
	dbInterface.exec( sql );

}
