/******************************************************************************
 *
 * Copyright (c) 2006 Jon Black
 *
 * This file is part of Gibberish.
 *
 * Gibberish is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * Gibberish is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *****************************************************************************/

#include <QSqlQuery>
#include <QSqlQueryModel>
#include <QDebug>
#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QFile>
#include <QSettings>
#include <QLabel>
#include <QFileDialog>
#include <QComboBox>
#include <QMessageBox>

#include "MainWindow.h"
#include "LessonSettings.h"
#include "Score.h"
#include "Questions.h"
#include "LessonDialog.h"
#include "Lesson.h"
#include "VocabularyParser.h"
#include "DatabaseInterface.h"
#include "Vocabulary.h"
#include "HelpAbout.h"
#include "LanguageMode.h"
#include "DatabaseInterface.h"

// *****************************************************************************
// Constructor/Destructor
// *****************************************************************************

/**
 *
 */
MainWindow::MainWindow( QWidget * parent, Qt::WFlags flags )
	: QMainWindow ( parent, flags )
{
	setupUi( this );

	// 1. Load settings from configuration file

	QSettings settings("gibberish.ini", QSettings::IniFormat);
	settings.beginGroup("StudentProfile");
	studentProfile.setForename( settings.value("forename", QString("unknown") ).toString() );
	studentProfile.setSurname( settings.value("surname", QString("unknown") ).toString() );
	QString entry = settings.value("nativelanguages", QString("") ).toString();
	studentProfile.setNativeLanguages( entry.split("+") );
	entry = settings.value("studylanguages", QString("") ).toString();
	studentProfile.setStudyLanguages( entry.split("+") );
	studentProfile.setCurrentNativeLanguage( settings.value("currentnativelanguage", QString("unknown") ).toString() );
	studentProfile.setCurrentStudyLanguage( settings.value("currentstudylanguage", QString("unknown") ).toString() );
	settings.endGroup();

	// 2. Show lessons in the list

	QString sql = QString( "SELECT Lessons.LessonID, Title, Author, Email, Created, XMLPath FROM Lessons, LessonLanguages, LanguageCodes WHERE Lessons.LessonID=LessonLanguages.LessonID AND LessonLanguages.LangID=LanguageCodes.LangID AND LanguageCodes.LangID='%1' ORDER BY Title;" ).arg(studentProfile.getCurrentStudyLanguage());
  filterLessons( sql );

	// 3. Show lessons taken

	sql = QString( "SELECT Lessons.LessonID, Title COUNT(TakenLessons.LessonID), QuestionCount FROM Lessons, TakenLessons, LessonLanguages WHERE Lessons.LessonID=TakenLessons.LessonID AND TakenLessons.LangID=LessonLanguages.LangID AND Lessons.LessonID=LessonLanguages.LessonID AND TakenLessons.LangID='%1' GROUP BY TakenLessons.LessonID;" ).arg(studentProfile.getCurrentStudyLanguage());
	filterLessonsTaken( sql );

	// 4. Setup display

	wLanguageMode = new QLabel( "mode" );
	statusBar()->addPermanentWidget( wLanguageMode );
	statusBar()->addWidget( new QLabel( "Ready!" ), 100 );
	wLessons->setColumnHidden( 0, true );	
	wLessons->setColumnHidden( 5, true );
	languageUpdate();
}

// *****************************************************************************
// Methods
// *****************************************************************************

/**
 *
 */
void MainWindow::on_wTakeLesson_clicked()
{

	// 1. Create the settings

	LessonSettings * lSettings = new LessonSettings;
	lSettings->setNativeLang( studentProfile.getCurrentNativeLanguage() );
	lSettings->setForeignLang( studentProfile.getCurrentStudyLanguage() );
	lSettings->setLessonType( wLessonType->currentIndex() );
	
	if ( wTimer->checkState() == Qt::Checked )
	{
		lSettings->setUseTimer( true );
		lSettings->setTime( wTime->time() );
	}
	else
	{
		lSettings->setUseTimer( false );
		lSettings->setTime( QTime( 0, 0 ) );
	}

	// 2. Parse the XML file

	VocabularyParser vParser = VocabularyParser( lSettings->getNativeLang(), lSettings->getStudyLang() );
	Questions * lQuestions = new Questions();

	QString xmlPath = wLessons->currentIndex().sibling( wLessons->currentIndex().row(), 5 ).data().toString();
	QFile file( xmlPath );
	
	if ( file.exists() )
	{
		vParser.parse( &file, lQuestions );

		// 3. Create the lesson
	
		Lesson * lesson = new Lesson();
		lesson->setTitle( wLessons->currentIndex().sibling( wLessons->currentIndex().row(), 1 ).data().toString() );
		lesson->setLessonSettings( lSettings );
		lesson->setQuestions( lQuestions );
		lesson->setLessonID( wLessons->currentIndex().sibling( wLessons->currentIndex().row(), 0 ).data().toInt() );
	
		// 4. Start the lesson
	
		LessonDialog* lessonDialog = new LessonDialog( 0, lesson );
		lessonDialog->exec();
	}
	else
	{
		//qDebug() << "Error: File does not exist"; /// \todo show a msg box here
		QMessageBox::critical( this, "Error", "Lesson vocabulary file not found" );
	}
}

/**
 *
 */
void MainWindow::on_actionAboutGibberish_triggered()
{
	HelpAbout * aboutDlg = new HelpAbout();
	aboutDlg->exec();
}

/**
 *
 */
void MainWindow::on_actionQuit_triggered()
{
	close();
}

/**
 *
 */
void MainWindow::on_wFilter_textChanged( const QString & text )
{
	QString filterCol = wFilterCol->currentText();

	QString filterText = wFilter->text();
	QString sql; 

	if ( wFilterCol->currentIndex() != 0 )
		sql = QString( "SELECT Lessons.LessonID, Title, Author, Email, Created, XMLPath FROM Lessons, LessonLanguages, LanguageCodes WHERE Lessons.LessonID=LessonLanguages.LessonID AND LessonLanguages.LangID=LanguageCodes.LangID AND LanguageCodes.LangID='%1'AND %2 LIKE '%3%' ORDER BY Title;" ).arg(studentProfile.getCurrentStudyLanguage()).arg(filterCol).arg(text);
	else
		sql = QString( "SELECT Lessons.LessonID, Title, Author, Email, Created, XMLPath FROM Lessons, LessonLanguages, LanguageCodes WHERE Lessons.LessonID=LessonLanguages.LessonID AND LessonLanguages.LangID=LanguageCodes.LangID AND LanguageCodes.LangID='%1' ORDER BY Title;" ).arg(studentProfile.getCurrentStudyLanguage());
	
	filterLessons( sql );
}

/**
 *
 */
void MainWindow::on_wFilterCol_currentIndexChanged( const QString & text )
{
	QString filterText = wFilter->text();
	QString sql; 

	if ( wFilterCol->currentIndex() != 0 )
		sql = QString( "SELECT Lessons.LessonID, Title, Author, Email, Created, XMLPath FROM Lessons, LessonLanguages, LanguageCodes WHERE Lessons.LessonID=LessonLanguages.LessonID AND LessonLanguages.LangID=LanguageCodes.LangID AND LanguageCodes.LangID='%1'AND %2 LIKE '%3%' ORDER BY Title;" ).arg(studentProfile.getCurrentStudyLanguage()).arg(text).arg(filterText);
	else
		sql = QString( "SELECT Lessons.LessonID, Title, Author, Email, Created, XMLPath FROM Lessons, LessonLanguages, LanguageCodes WHERE Lessons.LessonID=LessonLanguages.LessonID AND LessonLanguages.LangID=LanguageCodes.LangID AND LanguageCodes.LangID='%1' ORDER BY Title;" ).arg(studentProfile.getCurrentStudyLanguage());
	
	filterLessons( sql );
}

/**
 *
 */
void MainWindow::on_wLessons_clicked( const QModelIndex & )
{
	wTakeLesson->setEnabled( true );
}

/**
 *
 */
void MainWindow::on_wTimer_stateChanged( int state )
{
	if ( state == Qt::Checked )
		wTime->setEnabled( true );	
	else
		wTime->setEnabled( false );
}

/**
 *
 */
void MainWindow::on_actionLanguageMode_triggered()
{
	LanguageMode * uiLanguageMode = new LanguageMode( studentProfile );
	int rc = uiLanguageMode->exec();

	// 1. Was a change made? If so, save it.

	if ( rc == QDialog::Accepted )
	{
		studentProfile.setCurrentNativeLanguage( uiLanguageMode->getNativeLanguage() );
		studentProfile.setCurrentStudyLanguage( uiLanguageMode->getStudyLanguage() );
	}

	// 2. Update display

	languageUpdate();
}

void MainWindow::languageUpdate()
{
	// 1. Filter the lesson table

	QString sql = QString( "SELECT Lessons.LessonID, Title, Author, Email, Created, XMLPath FROM Lessons, LessonLanguages, LanguageCodes WHERE Lessons.LessonID=LessonLanguages.LessonID AND LessonLanguages.LangID=LanguageCodes.LangID AND LanguageCodes.LangID='%1' ORDER BY Title;" ).arg(studentProfile.getCurrentStudyLanguage());
  filterLessons( sql );

	// 2. Filter the lessons taken table

	sql = QString( "SELECT Lessons.LessonID, Title, COUNT(TakenLessons.LessonID), QuestionCount FROM Lessons, TakenLessons, LessonLanguages WHERE Lessons.LessonID=TakenLessons.LessonID AND TakenLessons.LangID=LessonLanguages.LangID AND Lessons.LessonID=LessonLanguages.LessonID AND TakenLessons.LangID='%1' GROUP BY TakenLessons.LessonID;" ).arg(studentProfile.getCurrentStudyLanguage());
	filterLessonsTaken( sql );
	
	// 3. Set the labels

	QString labelText = "<b>Language Mode:</b> " + studentProfile.getCurrentNativeLanguage() + " -> " + studentProfile.getCurrentStudyLanguage();
	wLanguageMode->setText( labelText );

	DatabaseInterface dbInterface;
	sql = QString( "SELECT name FROM LanguageCodes WHERE LangID='%1';" ).arg( studentProfile.getCurrentStudyLanguage() );
	QSqlQuery * query = dbInterface.exec( sql );

	if ( query->isActive() )
	{
		query->first();
		QString name = query->value(0).toString();
		wLessonLabel->setText( QString( "%1 Lessons" ).arg( name ) );
		wProgressLabel->setText( QString( "%1 Progress" ).arg( name ) );
	}
}

/**
 *
 */
void MainWindow::filterLessons( QString sql )
{
	QSqlQueryModel * model = new QSqlQueryModel;
	model->setQuery( sql );
  model->setHeaderData( 0, Qt::Horizontal, tr( "Lesson ID" ) );
  model->setHeaderData( 1, Qt::Horizontal, tr( "Title" ) );
  model->setHeaderData( 2, Qt::Horizontal, tr( "Author" ) );
  model->setHeaderData( 3, Qt::Horizontal, tr( "Email" ) );
  model->setHeaderData( 4, Qt::Horizontal, tr( "Created" ) );
  model->setHeaderData( 5, Qt::Horizontal, tr( "XMLPath" ) );
	wLessons->setModel( model );
}

/**
 *
 */
void MainWindow::filterLessonsTaken( QString sql )
{
	QSqlQueryModel * model = new QSqlQueryModel;
	model->setQuery( sql );
	model->setHeaderData(0, Qt::Horizontal, tr("Lesson ID"));
  model->setHeaderData(1, Qt::Horizontal, tr("Lesson"));
  model->setHeaderData(2, Qt::Horizontal, tr("Times Taken"));
  model->setHeaderData(3, Qt::Horizontal, tr("# of Questions"));
	wLessonsTakenTable->setModel( model );
  //wLessonsTakenTable->setColumnHidden( 0, true );
}

/**
 *
 */
void MainWindow::filterLessonResults( QString sql )
{
	QSqlQueryModel * model = new QSqlQueryModel;
	model->setQuery( sql );
	model->setHeaderData(0, Qt::Horizontal, tr("Result ID"));
  model->setHeaderData(1, Qt::Horizontal, tr("Date Taken"));
  model->setHeaderData(2, Qt::Horizontal, tr("Result"));
	wLessonResults->setModel( model );
	//wLessonResults->setColumnHidden( 0, true );
}

/**
 *
 */
void MainWindow::on_wLessonsTakenTable_clicked( const QModelIndex & mIndex )
{
	int row = mIndex.row();
	QString lessonID = mIndex.sibling( row, 0 ).data().toString();

	QString sql = QString( "SELECT TakenLessons.ResultID, DateTaken, SUM(Result) FROM TakenLessons, Results WHERE TakenLessons.ResultID=Results.ResultID AND TakenLessons.LessonID='%1' GROUP BY TakenLessons.ResultID ORDER BY DateTaken;" ).arg(lessonID);

	filterLessonResults( sql );

	// 2. Get the results from the list and show them in
	//    the graph

	QList<double> resultsList;

	int lessonResultsCount = wLessonResults->model()->rowCount();	
	for ( int i = 0; i <= lessonResultsCount - 1; i++ )
	{
		QModelIndex ffs = wLessonResults->model()->index( i, 2 );
		double score = wLessonResults->model()->data( ffs ).toDouble();

		//qDebug() << "Score:" << score;
		resultsList.append( score );
	}

	QModelIndex ffs2 = wLessonsTakenTable->model()->index( row, 3 );
	int maxy = wLessonsTakenTable->model()->data( ffs2 ).toInt();

	linechart->setXMaxVal( lessonResultsCount );
	linechart->setYMaxVal( maxy );
	linechart->setDataPointsList( resultsList );
}
